#include "MainWindow.hpp"
#include "DetailWindow.hpp"
#include "rssreader_sfy.hpp"
#include "rssreader_sfy.brh"
#include "Utility.hpp"
#include "ColorDef.hpp"
#include "SoftMenuID.hpp"

// RXgN^
DetailWindow::DetailWindow(Void) static_throws
{
    if (static_try()) {
        // L[nh̓o^
        static_throw(RegisterHandler(
            SFXEventRange(SFEVT_KEY, SFEVT_KEY, SFP16_BEGIN, SFP16_END),
            XANDLER_INTERNAL(OnKey)
        ));
        if (static_try()) {
            // \tgL[nh̓o^
            static_throw(RegisterHandler(
                SFXEventRange(SFEVT_RESPONDER_SOFTKEY, SFEVT_RESPONDER_SOFTKEY, AVK_SOFT1, AVK_SOFT4),
                XANDLER_INTERNAL(OnSoftKey)
            ));
            if (static_try()) {
                static_throw(SetSoftKey());
            }
        }
    }
}

// fXgN^
DetailWindow::~DetailWindow(Void)
{
}

// CX^X̍쐬
DetailWindowSmp DetailWindow::NewInstance(SFCErrorPtr exception)
{
    return static_pointer_cast<DetailWindow>(Factory(::new DetailWindow(), exception));
}

// X|_̍쐬Ɣzu
SFCError DetailWindow::Make(RSSFeedPtr feed, SInt32 cursor)
{
    SFCError error;

    _currentFeed = feed;
    _cursor = cursor;

    // XN[o[̍쐬 (t[e)
    SFZContainerScrollBarControlSmp bar = SFZContainerScrollBarControl::NewInstance(&error);
    if (bar != null) {
        if ((error = bar->SetParent(GetParent())) == SFERR_NO_ERROR) {
            bar->SetBevelColor(COLOR_SCROLL_BEVEL);
            bar->SetArrowColor(COLOR_SCROLL_ARROW);
            SFXRectangle barRect(GetRealBound());
            barRect.SetLeft(barRect.GetRight());
            barRect.SetRight(barRect.GetLeft() + SCROLLBAR_WIDTH);
            bar->SetRealBound(barRect);
            bar->SetState(true, true, false, false);
            bar->ToFront();
            error = bar->Bind(GetThis());
        }
    }
    // ݂̐x̍쐬
    if (error == SFERR_NO_ERROR) {
        _labelCurDesc = SFZMultipleTextLabelControl::NewInstance(&error);
        if (_labelCurDesc != null) {
            error = _labelCurDesc->RegisterHandler(
                SFXEventRange(SFEVT_RESPONDER_STYLE, SFEVT_RESPONDER_STYLE, SFP16_STYLE_TEXT, SFP16_STYLE_TEXT),
                XANDLER_INTERNAL(OnLabelText)
            );
            if (error == SFERR_NO_ERROR) {
                if ((error = _labelCurDesc->SetParent(GetThis())) == SFERR_NO_ERROR) {
                    error = _labelCurDesc->SetText(Util::DecodeHtml(_currentFeed->GetItem(_cursor)->description));
                    if (error == SFERR_NO_ERROR) {
                        _labelCurDesc->SetHorizontalAlign(SFZMultipleTextLabelControl::HORIZONTAL_LEFT);
                        _labelCurDesc->SetVerticalAlign(SFZMultipleTextLabelControl::VERTICAL_TOP);
                        _labelCurDesc->SetState(true, true, false, false);
                        _labelCurDesc->SetRealBound(GetLocalBound());
                        _labelCurDesc->SetBackgroundColor(COLOR_CONTENTS_BACK);
                    }
                }
            }
        }
    }
    if (error == SFERR_NO_ERROR) {
        // ǂ̃ACeǂɂ
        if (_currentFeed != null && _currentFeed->GetSize() > cursor) {
            _currentFeed->GetItem(cursor)->alreadyRead = true;
            rssreader_sfy::GetMainWindow()->Render();
        }
        _currentFeed->SaveToFile();
    }
    return error;
}

// \tgL[j[̐ݒ
SFCError DetailWindow::SetSoftKey(Void)
{
    SFZSoftKeyControlSmp soft;
    SFCError error(SFERR_NO_ERROR);

    if ((soft = rssreader_sfy::GetSoftKey()) != null) {
        UInt32 key_text[] = {IDS_SOFTKEY_RETURN, IDS_SOFTKEY_PREV, IDS_SOFTKEY_NEXT};
        if ((error = soft->CreateMenu(SOFTMENU_DETAIL_WINDOW)) == SFERR_NO_ERROR) {
            error = soft->SetTextKey(SOFTMENU_DETAIL_WINDOW, key_text, lengthof(key_text));
        }
        if (error == SFERR_NO_ERROR) {
            error = soft->Bind(this, SOFTMENU_DETAIL_WINDOW);
        }
    }
    else {
        error = SFERR_FAILED;
    }
    return error;
}

// et[̃^Cgݒ
SFCError DetailWindow::SetText(SFXWideStringConstRef text)
{
    SFZTitlePlainFrameSmp frame(static_pointer_cast<SFZTitlePlainFrame>(GetParent()));
    SFCError error(SFERR_NO_ERROR);

    if (frame != null) {
        error = frame->SetText(text);
    }
    else {
        error = SFERR_FAILED;
    }
    return error;
}

// ̍ڂ\
Void DetailWindow::ViewNextItem(Void)
{
    RSSFeedPtr current = rssreader_sfy::GetCurrentFeed();

    if (current != null) {
        if (current->GetSize() > 0) {
            if (_cursor > 0) {
                _cursor--;
            }
            else {
                _cursor = current->GetSize() - 1;
            }
            SetText((current != null) ? current->GetItem(_cursor)->title : "");
            _labelCurDesc->SetText(Util::DecodeHtml(current->GetItem(_cursor)->description));
        }
    }
    return;
}

// O̍ڂ\
Void DetailWindow::ViewPrevItem(Void)
{
    RSSFeedPtr current = rssreader_sfy::GetCurrentFeed();

    if (current != null) {
        if (current->GetSize() > 0) {
            if (_cursor < current->GetSize()-1) {
            _cursor++;
            }
            else {
                _cursor = 0;
            }
            SetText((current != null) ? current->GetItem(_cursor)->title : "");
            _labelCurDesc->SetText(Util::DecodeHtml(current->GetItem(_cursor)->description));
        }
    }
    return;
}

// L[nh
XANDLER_IMPLEMENT_BOOLEVENT(DetailWindow, OnKey, invoker, event)
{
    Bool result(false);
    unused(invoker);

    switch (event.GetP16()) {
        case AVK_CLR:
        case AVK_SELECT:
            GetParent()->Terminate();
            this->Terminate();
            result = true;
            break;
        case AVK_RIGHT:
            ViewNextItem();
            result = true;
            break;
        case AVK_LEFT:
            ViewPrevItem();
            result = true;
            break;
    }
    return result;
}

// \tgL[ nh
XANDLER_IMPLEMENT_VOIDRESULT(DetailWindow, OnSoftKey, invoker, reason, result)
{
    unused(invoker); unused(result);

    switch (reason) {
        case AVK_SOFT1: // O
            ViewNextItem();
            break;
        case AVK_SOFT2: // 
            ViewPrevItem();
            break;
    }
    return;
}

// xeLXg̃X^Cnh
XANDLER_IMPLEMENT_VOIDSTYLE(DetailWindow, OnLabelText, invoker, reason)
{
    SFXGraphicsPtr graphics(SFXGraphics::GetInstance());
    SFXSize realSize(GetRealBound().GetSize());
    SFXRectangle rectangle;
    unused(invoker); unused(reason);

    rectangle.Set(
        SFXGrid::ZeroInstance(),
        graphics->MeasureMultipleText(_labelCurDesc->GetText(), realSize.GetWidth())
    );
    if (realSize.GetHeight() > rectangle.GetHeight()) {
        rectangle = SFXRectangle(SFXGrid::ZeroInstance(), realSize);
    }
    SetVirtualBound(rectangle);
    _labelCurDesc->SetRealBound(rectangle);
    return;
}

