#include "InputWindow.hpp"
#include "rssreader_sfy.hpp"
#include "rssreader_sfy.brh"
#include "Utility.hpp"
#include "ColorDef.hpp"
#include "SoftMenuID.hpp"

#define TEXT_HTTP_HEADER                "http://"

// RXgN^
InputWindow::InputWindow(Void) static_throws
{
    static SFXEventRange::AtomRecConst  range[] = {
        {SFEVT_KEY, SFEVT_KEY, SFP16_BEGIN, SFP16_END},
        {SFEVT_RESPONDER_SOFTKEY, SFEVT_RESPONDER_SOFTKEY, AVK_SOFT1, AVK_SOFT2}
    };
    SFYHandler::RuleRec                 rule[lengthof(range)];

    if (static_try()) {
        rule[0].spp = XANDLER_FUNCTION(OnKey);      // L[ nh̓o^
        rule[0].reference = this;
        rule[1].spp = XANDLER_FUNCTION(OnSoftKey);  // \tgL[nh̓o^
        rule[1].reference = this;
        static_throw(RegisterHandler(atomic_cast(range), rule, lengthof(range)));
        if (static_try()) {
            static_throw(SetSoftKey());
            if (static_try()) {
                if ((_dialoger = rssreader_sfy::GetDialoger()) == null) {
                    static_throw(SFERR_FAILED);
                }
            }
        }
    }
}

// fXgN^
InputWindow::~InputWindow(Void)
{
}

// CX^X̍쐬
InputWindowSmp InputWindow::NewInstance(SFCErrorPtr exception)
{
    return static_pointer_cast<InputWindow>(Factory(::new InputWindow(), exception));
}

// X|_̍쐬
SFCError InputWindow::Make(Void)
{
    SFXRectangle rectangle(GetLocalBound());
    SFCError error;

    // tB[h̃xƃGfBbg{bNX쐬
    if ((error = CreateFeedTitle(rectangle.Deflate(X_MARGIN, Y_MARGIN))) == SFERR_NO_ERROR) {
        // tB[h URL ̃xƃGfBbg{bNXtB[h쐬
        error = CreateFeedUrl(rectangle);
    }
    return error;
}

// tB[h̃xƃGfBbg{bNX쐬
SFCError InputWindow::CreateFeedTitle(SFXRectangleRef rectangle)
{
    SFXRectangle rect;
    SInt16 width(rectangle.GetWidth());
    SInt16 height = SFXGraphics::GetFontHeight(AEE_FONT_NORMAL) + 4;
    SFCError error;

    // tB[h ^Cg x̍쐬
    if ((_labelTitle = SFZSingleTextLabelControl::NewInstance(&error)) != null) {
        if ((error = _labelTitle->SetParent(GetThis())) == SFERR_NO_ERROR) {
            if ((error = _labelTitle->SetText(rssreader_sfy::LoadString(IDS_LABEL_TITLE))) == SFERR_NO_ERROR) {
                _labelTitle->SetHorizontalAlign(SFZSingleTextLabelControl::HORIZONTAL_LEFT);
                rect = SFXRectangle(rectangle.GetOrigin(), width, height);
                _labelTitle->SetRealBound(rect);
                _labelTitle->SetState(true, true, false, false);
                rectangle.SetTop(rect.GetBottom());
            }
        }
    }
    if (error == SFERR_NO_ERROR) {
        // tB[h ^Cg GfBbg{bNX̍쐬
        if ((_editboxTitle = SFZSingleEditBoxControl::NewInstance(&error)) != null) {
            if ((error = _editboxTitle->SetParent(GetThis())) == SFERR_NO_ERROR) {
                _editboxTitle->SetText("");
                _editboxTitle->SetHorizontalAlign(SFZSingleEditBoxControl::HORIZONTAL_LEFT);
                rect = SFXRectangle(rectangle.GetOrigin(), width, height);
                _editboxTitle->SetRealBound(rect);
                _editboxTitle->SetState(true, true, true, true);
                rectangle.SetTop(rect.GetBottom());
            }
        }
    }
    return error;
}

// tB[h URL ̃xƃGfBbg{bNXtB[h쐬
SFCError InputWindow::CreateFeedUrl(SFXRectangleRef rectangle)
{
    SFXRectangle rect;
    SInt16 width(rectangle.GetWidth());
    SInt16 height = SFXGraphics::GetFontHeight(AEE_FONT_NORMAL) + 4;
    SFCError error;

    // tB[h URL x̍쐬
    if ((_labelURL = SFZSingleTextLabelControl::NewInstance(&error)) != null) {
        if ((error = _labelURL->SetParent(GetThis())) == SFERR_NO_ERROR) {
            if ((error = _labelURL->SetText(rssreader_sfy::LoadString(IDS_LABEL_URL))) == SFERR_NO_ERROR) {
                _labelURL->SetHorizontalAlign(SFZSingleTextLabelControl::HORIZONTAL_LEFT);
                rect = SFXRectangle(rectangle.GetOrigin(), width, height);
                _labelURL->SetRealBound(rect);
                _labelURL->SetState(true, true, false, false);
                rectangle.SetTop(rect.GetBottom());
            }
        }
    }
    if (error == SFERR_NO_ERROR) {
        // tB[h URL GfBbg{bNX̍쐬
        if ((_editboxURL = SFZSingleEditBoxControl::NewInstance(&error)) != null) {
            if ((error = _editboxURL->SetParent(GetThis())) == SFERR_NO_ERROR) {
                if ((error = _editboxURL->SetText(TEXT_HTTP_HEADER)) == SFERR_NO_ERROR) {
                    _editboxURL->SetInputMode(AEE_TM_LETTERS);
                    _editboxURL->SetHorizontalAlign(SFZSingleEditBoxControl::HORIZONTAL_LEFT);
                    rect = SFXRectangle(rectangle.GetOrigin(), width, height);
                    _editboxURL->SetRealBound(rect);
                    _editboxURL->SetState(true, true, true, false);
                    rectangle.SetTop(rect.GetBottom());
                }
            }
        }
    }
    return error;
}

// GfBbg{bNX͓̓eȒPɃ`FbN
SFCError InputWindow::CheckContents(Void)
{
    SFCError error(SFERR_NO_ERROR);

    if (_editboxTitle->GetText().IsEmpty()) {
        _editboxTitle->SetStateFocus(true);
        error = SFERR_FAILED;
    }
    else if (_editboxURL->GetText() == TEXT_HTTP_HEADER) {
        _editboxURL->SetStateFocus(true);
        error = SFERR_FAILED;
    }
    if (error == SFERR_FAILED) {
        _dialoger->SetTitle(Dialoger::ERROR);
        _dialoger->SetMessageText(rssreader_sfy::LoadString(IDS_DIALOG_ERROR_EDITBOX));
        _dialoger->Execute();
    }
    return error;
}

// \tgL[j[̐ݒ
SFCError InputWindow::SetSoftKey(Void)
{
    SFZSoftKeyControlSmp soft;
    SFCError error(SFERR_NO_ERROR);

    if ((soft = rssreader_sfy::GetSoftKey()) != null) {
        UInt32 text_key[] = {IDS_SOFTKEY_SELECT, IDS_SOFTKEY_RETURN, IDS_SOFTKEY_REGISTER};
        if (!soft->ContainsMenuKey(SOFTMENU_INPUT_WINDOW)) {
            if ((error = soft->CreateMenu(SOFTMENU_INPUT_WINDOW)) == SFERR_NO_ERROR) {
                error = soft->SetTextKey(SOFTMENU_INPUT_WINDOW, text_key, lengthof(text_key));
            }
        }
        if (error == SFERR_NO_ERROR) {
            error = soft->Bind(this, SOFTMENU_INPUT_WINDOW);
        }
    }
    else {
        error = SFERR_FAILED;
    }
    return error;
}

// mF_CAOIƂ̃nh
XANDLER_IMPLEMENT_VOIDRESULT(InputWindow, OnFinish, invoker, reason, result)
{
    unused(invoker); unused(reason); unused(result);

    _dialoger->Terminate();
    GetParent()->Terminate();
    this->Terminate();
    return;
}

// L[nh
XANDLER_IMPLEMENT_BOOLEVENT(InputWindow, OnKey, invoker, event)
{
    unused(invoker);

    if (event.GetP16() == AVK_CLR) {
        GetParent()->Terminate();
        this->Terminate();
        return true;
    }
    return false;
}

// \tgL[nh
XANDLER_IMPLEMENT_VOIDEVENT(InputWindow, OnSoftKey, invoker, event)
{
    unused(invoker);
    RSSFeedPtr feed;
    SFCError error;

    switch (event.GetP16()) {
        case AVK_SOFT1:
            GetParent()->Terminate();
            this->Terminate();
            break;
        case AVK_SOFT2:
            if ((error = CheckContents()) == SFERR_NO_ERROR) {
                // tB[h̍쐬
                if ((feed = ::new RSSFeed(_editboxTitle->GetText(), _editboxURL->GetText())) != null) {
                    if (rssreader_sfy::GetRSSFeedList()->Add(feed)) {
                        // ǉf[^ۑ
                        rssreader_sfy::GetRSSFeedList()->SaveToFile();
                    }
                    else {
                        ::delete feed;
                        feed = null;
                    }
                }
                // tB[h̍쐬ɐꍇ
                if (feed != null) {
                    _dialoger->SetTitle(Dialoger::CONFIRM);
                    _dialoger->SetMessageText(rssreader_sfy::LoadString(IDS_DIALOG_REGISTERED));
                    _dialoger->RegisterHandler(XANDLER_INTERNAL(OnFinish));
                    error = _dialoger->Execute();
                }
            }
            break;
    }
    return;
}
