#include "MainWindow.hpp"
#include "DetailWindow.hpp"
#include "rssreader_sfy.hpp"
#include "rssreader_sfy.brh"
#include "Utility.hpp"
#include "ColorDef.hpp"
#include "SoftMenuID.hpp"

#define COLOR_MAIN_LIST_BASE            COLOR_ELE_GLAY
#define COLOR_MAIN_LIST_SELB            COLOR_ELE_RED
#define COLOR_MAIN_LIST_SELF            COLOR_WHITE
#define COLOR_MAIN_LIST_ODDLINE_BACK    COLOR_ELE_SKYBLUE
#define COLOR_MAIN_LIST_EVENLINE_BACK   COLOR_ELE_SKYGREEN

// RXgN^
MainWindow::MainWindow(Void) static_throws
{
    if (static_try()) {
        // \tgL[nh̓o^
        static_throw(RegisterHandler(
            SFXEventRange(SFEVT_RESPONDER_SOFTKEY, SFEVT_RESPONDER_SOFTKEY, AVK_SOFT1, AVK_SOFT4),
            XANDLER_INTERNAL(OnSoftKey)
        ));
        if (static_try()) {
            // \tgL[̐ݒ
            static_throw(SetSoftKey());
        }
    }
}

// fXgN^
MainWindow::~MainWindow(Void)
{
}

// CX^X̍쐬
MainWindowSmp MainWindow::NewInstance(SFCErrorPtr exception)
{
    return static_pointer_cast<MainWindow>(Factory(::new MainWindow(), exception));
}

// X|_̍쐬
SFCError MainWindow::Make(Void)
{
    SFXRectangle rectangle(GetLocalBound());
    SFCError error;

    _currentFeed = rssreader_sfy::GetCurrentFeed();
    if ((error = SetText((_currentFeed != null) ? _currentFeed->_title : "No Feed")) == SFERR_NO_ERROR) {
        if ((error = CreateItemTitles(rectangle)) == SFERR_NO_ERROR) {
            if ((error = CreateDescription(rectangle)) == SFERR_NO_ERROR) {
                Update();
            }
        }
    }
    return error;
}

// tB[hACeXg̍쐬Ɣzu
SFCError MainWindow::CreateItemTitles(SFXRectangleRef rectangle)
{
    static SFXEventRange::AtomRecConst  range[] = {
        {SFEVT_RESPONDER_VALUE, SFEVT_RESPONDER_VALUE,  SFP16_VALUE_MAXIMUM, SFP16_VALUE_CURRENT},
        {SFEVT_RESPONDER_RESULT,SFEVT_RESPONDER_RESULT, SFP16_BEGIN,         SFP16_END}
    };
    SFYHandler::RuleRec                 rule[lengthof(range)];
    SFXRectangle                        rect(rectangle);
    SFCError                            error;

    // ^Cg Xg̃CX^X쐬
    if ((_listItemTitle = SFZListBoxControl::NewInstance(&error)) != null) {
        if ((error = _listItemTitle->SetParent(GetThis())) == SFERR_NO_ERROR) {
            rule[0].spp = XANDLER_FUNCTION(OnListValue);    // f[^Xglnh̓o^
            rule[0].reference = this;
            rule[1].spp = XANDLER_FUNCTION(OnSelect);       // XgInh̓o^
            rule[1].reference = this;
            if ((error = _listItemTitle->RegisterHandler(atomic_cast(range), rule, lengthof(range))) == SFERR_NO_ERROR) {
                _listItemTitle->SetBackgroundColor(COLOR_CONTENTS_BACK);            // wiFݒ
                _listItemTitle->SetBevelColor(SFXBevelColor(COLOR_MAIN_LIST_BASE)); // ׃x J[ݒ
                _listItemTitle->SetSelBackColor(COLOR_MAIN_LIST_SELB);              // Is̔wiFݒ
                _listItemTitle->SetSelForeColor(COLOR_MAIN_LIST_SELF);              // Is̃eLXgFݒ
                _listItemTitle->SetEscapeKey(AVK_SELECT);                           // ESCAPE L[ZNgɐݒ
                rect.SetHeight(_listItemTitle->GetItemHeight() * 5);                //
                _listItemTitle->SetRealBound(rect);                                 // Xg{bNX̍Tڕɐݒ
                _listItemTitle->SetState(true, true, true, true);                   // Ԃ (true, true, true, true)
                SFYScrollBarControlSmp bar = _listItemTitle->GetScrollBarControl();
                if (bar != null) {
                    bar->SetBevelColor(COLOR_SCROLL_BEVEL);                         // XN[o[̃xxFݒ
                    bar->SetArrowColor(COLOR_SCROLL_ARROW);                         // XN[o[̖Fݒ
                }
            }
        }
    }
    // Xg{bNẌ
    rectangle.SetTop(rect.GetBottom());
    return error;
}

// EBhE̍쐬Ɣzu
SFCError MainWindow::CreateDescription(SFXRectangleRef rectangle)
{
    SFXRectangle rect(rectangle);
    SFCError error;

    // EBhẼCX^X쐬
    if ((_windowDesc = SFZWindow::NewInstance(&error)) != null) {
        if ((error = _windowDesc->SetParent(GetThis())) == SFERR_NO_ERROR) {
            _windowDesc->SetBackgroundColor(COLOR_CONTENTS_BACK);
            _windowDesc->SetRealBound(rect);
            _windowDesc->SetState(true, true, false, false);
        }
    }
    if (error == SFERR_NO_ERROR) {
        // ݂̐x̃CX^X쐬
        if ((_labelCurDesc = SFZMultipleTextLabelControl::NewInstance(&error)) != null) {
            if ((error = _labelCurDesc->SetParent(_windowDesc)) == SFERR_NO_ERROR) {
                if (_currentFeed != null && _currentFeed->GetSize() > 0) {
                    error = _labelCurDesc->SetText(Util::DecodeHtml(_currentFeed->GetItem(0)->description));
                }
                else {
                    error = _labelCurDesc->SetText("No Item");
                }
                if (error == SFERR_NO_ERROR) {
                    _labelCurDesc->SetHorizontalAlign(SFZMultipleTextLabelControl::HORIZONTAL_LEFT);
                    _labelCurDesc->SetVerticalAlign(SFZMultipleTextLabelControl::VERTICAL_TOP);
                    _labelCurDesc->SetRealBound(_windowDesc->GetLocalBound());
                    _labelCurDesc->SetState(true, true, false, false);
                    _labelCurDesc->ToFront();
                }
            }
        }
    }
    rectangle.SetTop(rect.GetBottom());
    return error;
}

// \tgL[ nh
XANDLER_IMPLEMENT_VOIDRESULT(MainWindow, OnSoftKey, invoker, reason, result)
{
    SFCError error;
    unused(result); unused(invoker);

    switch (reason) {
        case AVK_SOFT1:     // \tgL[P (I)
            SFYApplication::Terminate();
            break;
        case AVK_SOFT2:     // \tgL[Q (Feed)
            if ((error = CreateFrame(rssreader_sfy::LoadString(IDS_TITLE_FEEDLIST))) == SFERR_NO_ERROR) {
                error = CreateSubscriptionWindow();
            }
            break;
    }
    return;
}

// Inh
XANDLER_IMPLEMENT_VOIDRESULT(MainWindow, OnSelect, invoker, reason, result)
{
    SFCError error;
    unused(invoker); unused(reason); unused(result);

    if (_listItemTitle->GetItemCount() > 0) {
        RSSItemPtr item = _currentFeed->GetItem(result);
        if (item != null) {
            if ((error = CreateFrame(item->title)) == SFERR_NO_ERROR) {
                error = CreateDetailWindow();
            }
        }
    }
}

// f[^Xglnh
XANDLER_IMPLEMENT_VOIDVALUE(MainWindow, OnListValue, invoker, reason, value)
{
    unused(invoker);
    switch (reason) {
        case SFP16_VALUE_CURRENT:
            if (_currentFeed != null) {
                RSSItemPtr item = _currentFeed->GetItem(value);
                if (item != null) {
                    _labelCurDesc->SetText(item->description);
                    // ǂ̌𑾎ŕ\B
                    _labelCurDesc->SetFont((item->alreadyRead) ? AEE_FONT_NORMAL : AEE_FONT_BOLD);
                }
            }
            break;
        case SFP16_VALUE_MAXIMUM:
            _soft->SetColorKey(SOFTMENU_MAIN_WINDOW, SFZSoftKeyControl::SELECT, (value == 0) ? KEY_DISABLE_COLOR : 0);
            break;
    }
}

// ʂXV
Void MainWindow::Update(Void)
{
    _currentFeed = rssreader_sfy::GetCurrentFeed();
    if ((_currentFeed != null) && (_currentFeed->GetSize() > 0)) {
        // XgĐ
        _listItemTitle->Clear();
        for (SIntN i = 0;  i< _currentFeed->GetSize(); ++i) {
            if (_currentFeed != null && i < _currentFeed->GetSize()) {
                RSSItemPtr item = _currentFeed->GetItem(i);
                if (item != null) {
                    _listItemTitle->InsertLast((!item->title.IsEmpty()) ? SFXWideString(item->title) : rssreader_sfy::LoadString(IDS_LIST_NONTITLE));
                    // sƋs̍ڂ̔wiFݒ
                    _listItemTitle->SetItemBackColor(i, (i % 2) ? COLOR_MAIN_LIST_ODDLINE_BACK : COLOR_MAIN_LIST_EVENLINE_BACK);
                    // ǁAǂŕ̑ύX
                    _labelCurDesc->SetFont((item->alreadyRead) ? AEE_FONT_NORMAL : AEE_FONT_BOLD);
                }
            }
        }
        if (_listItemTitle->GetCurrentValue() >= 0) {
            _labelCurDesc->SetText(_currentFeed->GetItem(_listItemTitle->GetCurrentValue())->description);
        }
    }
    else {
        _listItemTitle->Clear();
        _labelCurDesc->SetText("");
    }
    SetText((_currentFeed != null) ? _currentFeed->_title : "");
    return;
}

// t[̍쐬Ɣzu
SFCError MainWindow::CreateFrame(SFXWideStringConstRef title)
{
    SFCError error;

    if ((_frame = SFZTitlePlainFrame::NewInstance(&error)) != null) {
        if ((error = _frame->SetParent(GetRoot())) == SFERR_NO_ERROR) {
            if ((error = _frame->SetText(title)) == SFERR_NO_ERROR) {
                _frame->SetHeaderColor(COLOR_TITLE_BACK);
                _frame->SetTextColor(COLOR_TITLE_FORE);
                _frame->SetRealBound(rssreader_sfy::GetContentRect());
                _frame->SetState(true, true, true, true);
                _frame->ToFront();
            }
        }
    }
    return error;
}

// tB[hǗEBhE̍쐬
SFCError MainWindow::CreateSubscriptionWindow(Void)
{
    SFCError error;

    if ((_subscriptionWindow = SubscriptionWindow::NewInstance(&error)) != null) {
        if ((error = _subscriptionWindow->SetParent(_frame)) == SFERR_NO_ERROR) {
            _subscriptionWindow->SetRealBound(_frame->GetLocalBound().Deflate(_frame->GetSuitableMargin()));
            _subscriptionWindow->SetState(true, true, true, true);
            if ((error = _subscriptionWindow->Make()) == SFERR_NO_ERROR) {
            }
            else {
                _frame->Terminate();
            }
        }
    }
    return error;
}

// ڍ׃EBhE̍쐬
SFCError MainWindow::CreateDetailWindow(Void)
{
    SFCError error;

    if ((_detailWindow = DetailWindow::NewInstance(&error)) != null) {
        if ((error = _detailWindow->SetParent(_frame)) == SFERR_NO_ERROR) {
            _detailWindow->SetBackgroundColor(COLOR_CONTENTS_BACK);
            SFXRectangle rectangle(_frame->GetLocalBound().Deflate(_frame->GetSuitableMargin()));
            rectangle.SetWidth(rectangle.GetWidth() - DetailWindow::SCROLLBAR_WIDTH);
            _detailWindow->SetRealBound(rectangle);
            if ((error = _detailWindow->Make(_currentFeed, _listItemTitle->GetCurrentValue())) == SFERR_NO_ERROR) {
                _detailWindow->SetState(true, true, true, true);
            }
            else {
                _frame->Terminate();
            }
        }
    }
    return error;
}

// \tgL[j[̐ݒ
SFCError MainWindow::SetSoftKey(Void)
{
    SFCError error(SFERR_NO_ERROR);

    if ((_soft = rssreader_sfy::GetSoftKey()) != null) {
        if (!_soft->ContainsMenuKey(SOFTMENU_MAIN_WINDOW)) {
            UInt32 key_text[] = {IDS_SOFTKEY_DETAIL, IDS_SOFTKEY_EXIT, IDS_SOFTKEY_LIST};
            if ((error = _soft->CreateMenu(SOFTMENU_MAIN_WINDOW)) == SFERR_NO_ERROR) {
                if ((error = _soft->SetTextKey(SOFTMENU_MAIN_WINDOW, key_text, lengthof(key_text))) == SFERR_NO_ERROR) {
                    error = _soft->SetColorKey(SOFTMENU_MAIN_WINDOW, SFZSoftKeyControl::SELECT, KEY_DISABLE_COLOR);
                }
            }
        }
        if (error == SFERR_NO_ERROR) {
            error = _soft->Bind(this, SOFTMENU_MAIN_WINDOW);
        }
    }
    else {
        error = SFERR_FAILED;
    }
    return error;
}

// et[̃^Cgݒ
SFCError MainWindow::SetText(SFXWideStringConstRef text)
{
    SFZTitlePlainFrameSmp frame(static_pointer_cast<SFZTitlePlainFrame>(GetParent()));
    SFCError error(SFERR_NO_ERROR);

    if (frame != null) {
        error = frame->SetText(text);
    }
    else {
        error = SFERR_FAILED;
    }
    return error;
}
