#include "SubscriptionWindow.hpp"
#include "MainWindow.hpp"
#include "InputWindow.hpp"
#include "rssreader_sfy.hpp"
#include "rssreader_sfy.brh"
#include "Utility.hpp"
#include "ColorDef.hpp"
#include "SoftMenuID.hpp"

#define COLOR_SUBS_LIST_ODDLINE_BACK    COLOR_ELE_SKYBLUE
#define COLOR_SUBS_LIST_EVENLINE_BACK   COLOR_ELE_SKYGREEN
#define COLOR_URL_LIST_BASE             COLOR_ELE_GLAY
#define COLOR_URL_LIST_SELB             COLOR_ELE_RED

// RXgN^
SubscriptionWindow::SubscriptionWindow(Void) static_throws
{
    static SFXEventRange::AtomRecConst  range[] = {
        {SFEVT_KEY,                 SFEVT_KEY,                  SFP16_BEGIN,    SFP16_END},
        {SFEVT_RESPONDER_SOFTKEY,   SFEVT_RESPONDER_SOFTKEY,    AVK_SOFT1,      AVK_SOFT2}
    };
    SFYHandler::RuleRec                 rule[lengthof(range)];

    // L[nho^
    if (static_try()) {
        rule[0].spp = XANDLER_FUNCTION(OnKey);      // L[nh
        rule[0].reference = this;
        rule[1].spp = XANDLER_FUNCTION(OnSoftKey);  // \tgL[nh
        rule[1].reference = this;
        static_throw(RegisterHandler(atomic_cast(range), rule, lengthof(range)));
        if (static_try()) {
            static_throw(SetSoftKey());
            if (static_try()) {
                if ((_dialoger = rssreader_sfy::GetDialoger()) == null) {
                    static_throw(SFERR_FAILED);
                }
            }
        }
        else {
            static_throw(SFERR_FAILED);
        }
    }
    if (static_try()) {
        // J[\ʒuݒ
        _cursor = rssreader_sfy::GetRSSFeedList()->FirstIndexOf(rssreader_sfy::GetCurrentFeed());
    }
}

// fXgN^
SubscriptionWindow::~SubscriptionWindow(Void)
{
}

// CX^X쐬
SubscriptionWindowSmp SubscriptionWindow::NewInstance(SFCErrorPtr exception)
{
    return static_pointer_cast<SubscriptionWindow>(Factory(::new SubscriptionWindow, exception));
}

// qX|_̍쐬
SFCError SubscriptionWindow::Make(Void)
{
    SFXRectangle rectangle(GetLocalBound().Deflate(GetThis()->GetSuitableMargin()));
    SFCError error;

    if ((error = CreateFeedList(rectangle)) == SFERR_NO_ERROR) {
        if ((error = CreateUrlWindow(rectangle)) == SFERR_NO_ERROR) {
            Update();
        }
    }
    return error;
}

// \tgj[̐ݒ
SFCError SubscriptionWindow::SetSoftKey(Void)
{
    SFCError error(SFERR_NO_ERROR);

    if ((_soft = rssreader_sfy::GetSoftKey()) != null) {
        if (!_soft->ContainsMenuKey(SOFTMENU_SUBSCRIPTION_WINDOW)) {
            UInt32 key_text[] = {IDS_SOFTKEY_SELECT, IDS_SOFTKEY_RETURN, IDS_SOFTKEY_ADD};
            if ((error = _soft->CreateMenu(SOFTMENU_SUBSCRIPTION_WINDOW)) == SFERR_NO_ERROR) {
                error = _soft->SetTextKey(SOFTMENU_SUBSCRIPTION_WINDOW, key_text, lengthof(key_text));
            }
        }
        if (error == SFERR_NO_ERROR) {
            error = _soft->Bind(this, SOFTMENU_SUBSCRIPTION_WINDOW);
        }
    }
    else {
        error = SFERR_FAILED;
    }
    return error;
}

// tB[hXg̍쐬Ɣzu
SFCError SubscriptionWindow::CreateFeedList(SFXRectangleRef rectangle)
{
    SFXRectangle rect(rectangle);
    SFCError error(SFERR_NO_ERROR);

    // URL Xg̍쐬
    if ((_listURL = SFZListBoxControl::NewInstance(&error)) != null) {
        if ((error = _listURL->SetParent(GetThis())) == SFERR_NO_ERROR) {
            error = _listURL->RegisterHandler(
                SFXEventRange(SFEVT_RESPONDER_VALUE, SFEVT_RESPONDER_VALUE, SFP16_VALUE_MAXIMUM, SFP16_VALUE_CURRENT),
                XANDLER_INTERNAL(OnListValue)
            );
            if (error == SFERR_NO_ERROR) {
                error = _listURL->RegisterHandler(
                    SFXEventRange(SFEVT_RESPONDER_RESULT, SFEVT_RESPONDER_RESULT, SFP16_BEGIN, SFP16_END),
                    XANDLER_INTERNAL(OnSelect)
                );
                if (error == SFERR_NO_ERROR) {
                    // wiF̐ݒ
                    _listURL->SetBackgroundColor(COLOR_CONTENTS_BACK);

                    // ׃x J[ݒ
                    _listURL->SetBevelColor(SFXBevelColor(COLOR_URL_LIST_BASE));

                    // Is̔wiFݒ
                    _listURL->SetSelBackColor(COLOR_URL_LIST_SELB);

                    // ESCAPE L[ZNgɐݒ
                    _listURL->SetEscapeKey(AVK_SELECT);

                    rect.SetHeight(_listURL->GetItemHeight() * 7);
                    _listURL->SetRealBound(rect);

                    _listURL->SetState(true,true,true,true);
                }
            }
        }
    }
    rectangle.SetTop(rect.GetBottom());
    return error;
}

// URL EBhE̍쐬Ɣzu
SFCError SubscriptionWindow::CreateUrlWindow(SFXRectangleRef rectangle)
{
    SFXRectangle rect(rectangle);
    SFXAnsiString buff;
    SFCError error(SFERR_NO_ERROR);

    // URL EBhE̍쐬
    _windowURL = SFZWindow::NewInstance(&error);
    if (_windowURL != null) {
        error = _windowURL->SetParent(GetThis());
        if (error == SFERR_NO_ERROR) {
            _windowURL->SetBackgroundColor(COLOR_CONTENTS_BACK);
            _windowURL->SetRealBound(rect);
            _windowURL->SetState(true, true, false, false);
        }
    }
    if (error == SFERR_NO_ERROR) {
        RSSFeedListPtr list = rssreader_sfy::GetRSSFeedList();
        if (list != null && list->GetSize() > 0) {
            RSSFeedPtr item = (*list)[_listURL->GetCurrentValue()];
            if (item != null) {
                buff = "URL :" + item->_url;
            }
        }
        else {
            buff = "URL :";
        }
    }
    if (error == SFERR_NO_ERROR) {
        // URL eLXgx̍쐬
        if ((_labelURL = SFZMultipleTextLabelControl::NewInstance(&error)) != null) {
            if ((error = _labelURL->SetParent(_windowURL)) == SFERR_NO_ERROR) {
                if ((error = _labelURL->SetText(buff)) == SFERR_NO_ERROR) {
                    _labelURL->SetHorizontalAlign(SFZMultipleTextLabelControl::HORIZONTAL_LEFT);
                    _labelURL->SetVerticalAlign(SFZMultipleTextLabelControl::VERTICAL_TOP);
                    _labelURL->SetRealBound(_windowURL->GetLocalBound());
                    _labelURL->SetState(true, true, false, false);
                }
            }
        }
    }
    rectangle.SetTop(rect.GetBottom());
    return error;
}

// t[̍쐬Ɣzu
SFCError SubscriptionWindow::CreateFrame(Void)
{
    SFCError error;

    if ((_frame = SFZTitlePlainFrame::NewInstance(&error)) != null) {
        if ((error = _frame->SetParent(GetRoot())) == SFERR_NO_ERROR) {
            if ((error = _frame->SetText(rssreader_sfy::LoadString(IDS_TITLE_INPUT))) == SFERR_NO_ERROR) {
                _frame->SetHeaderColor(COLOR_TITLE_BACK);
                _frame->SetTextColor(COLOR_TITLE_FORE);
                _frame->SetRealBound(rssreader_sfy::GetContentRect());
                _frame->SetState(true, true, true, true);
                _frame->ToFront();
            }
        }
    }
    return error;
}

// ̓EBhE̍쐬Ɣzu
SFCError SubscriptionWindow::CreateInputWindow(Void)
{
    SFCError error;

    if ((_inputWindow = InputWindow::NewInstance(&error)) != null) {
        if ((error = _inputWindow->SetParent(_frame)) == SFERR_NO_ERROR) {
            error = _inputWindow->RegisterHandler(
                SFXEventRange(SFEVT_RESPONDER_STATE, SFEVT_RESPONDER_STATE, SFP16_STATE_VALID, SFP16_STATE_VALID),
                XANDLER_INTERNAL(OnInputWindowValid)
            );
            if (error == SFERR_NO_ERROR) {
                _inputWindow->SetBackgroundColor(COLOR_CONTENTS_BACK);
                _inputWindow->SetRealBound(_frame->GetLocalBound().Deflate(_frame->GetSuitableMargin()));
                _inputWindow->SetState(true, true, true, true);
                _inputWindow->Make();
            }
        }
    }
    return error;
}

// Tuj[̍쐬Ɣzu
SFCError SubscriptionWindow::MakeMenu(Void)
{
    SFZTextMenuSmp menu;
    SFCError error;

    if ((menu = SFZTextMenu::NewInstance(&error)) != null) {
        if ((menu->SetParent(GetRoot())) == SFERR_NO_ERROR) {
            error = menu->RegisterHandler(
                SFXEventRange(SFEVT_RESPONDER_RESULT, SFEVT_RESPONDER_RESULT, SFP16_RESULT_OK, SFP16_RESULT_OK),
                XANDLER_INTERNAL(OnMenuResult)
            );
            if (error == SFERR_NO_ERROR) {
                menu->SetTitle("Menu");
                menu->AppendItem(rssreader_sfy::LoadString(IDS_SOFTKEY_ADD), AVK_1, WChar('1'));
                if (_listURL->GetItemCount() > 0) {
                    menu->AppendItem(rssreader_sfy::LoadString(IDS_SOFTKEY_UPDATE), AVK_2, WChar('2'));
                    menu->AppendItem(rssreader_sfy::LoadString(IDS_SOFTKEY_DELETE), AVK_3, WChar('3'));
                }
                menu->SetBackgroundColor(COLOR_TITLE_BACK);
                menu->SetTitleBackColor(COLOR_TITLE_BACK);
                menu->SetTitleForeColor(COLOR_TITLE_FORE);
                menu->SetSelBackColor(COLOR_URL_LIST_SELB);
                SFXGrid rightBottom(_soft->GetRealBound().GetRightTop());
                menu->SetRealBound(SFXRectangle(SFXGrid::ZeroInstance(), 100, menu->GetItemHeight() * menu->GetItemCount() + menu->GetTitleHeight()).SnapRightBottom(rightBottom));
                _soft->Bind(menu.Get(), SOFTMENU_SELECT);
                menu->SetState(true, true, true, true);
                menu->ToFront();
            }
        }
    }
    return error;
}

// XgXV
Void SubscriptionWindow::Update(Void)
{
    RSSFeedListPtr list = rssreader_sfy::GetRSSFeedList();

    _listURL->Clear();
    _labelURL->SetText("");
    if ((list != null) && (list->GetSize() > 0)) {
        for (SIntN i = 0;  i < list->GetSize(); ++i) {
            SFXWideString title = (*list)[i]->_title;
            _listURL->InsertLast(!title.IsEmptyCString() ? title : rssreader_sfy::LoadString(IDS_LIST_NONTITLE));
            _listURL->SetItemBackColor(i, (i % 2) ? COLOR_SUBS_LIST_ODDLINE_BACK : COLOR_SUBS_LIST_EVENLINE_BACK);
        }
        _cursor = 0;
        _labelURL->SetText("URL: " + (*list)[_cursor]->_url);
        _listURL->SetCurrentValue(_cursor);
    }
    else {
        _cursor = -1;
    }
}

// ̓EBhE̗Lԃnh
XANDLER_IMPLEMENT_VOIDSTATE(SubscriptionWindow, OnInputWindowValid, invoker, reason, state)
{
    unused(invoker);
    unused(reason);

    if (!state) {
        Update();
    }
    return;
}

// L[nh
XANDLER_IMPLEMENT_BOOLEVENT(SubscriptionWindow, OnKey, invoker, event)
{
    RSSFeedListPtr feedList = rssreader_sfy::GetRSSFeedList();

    unused(invoker);
    switch (event.GetP16()) {
        case AVK_CLR:   // NAL[: EBhE
            if (feedList != null && _cursor >= 0) {
                rssreader_sfy::SetCurrentFeed((feedList->GetSize() > _cursor) ? ((*feedList)[_cursor]) : null);
                rssreader_sfy::GetMainWindow()->Update();
            }
            GetParent()->Terminate();
            this->Terminate();
            return true;
    }
    return false;
}

// \tgL[ nh
XANDLER_IMPLEMENT_VOIDRESULT(SubscriptionWindow, OnSoftKey, invoker, reason, result)
{
    RSSFeedListPtr feedList = rssreader_sfy::GetRSSFeedList();
    unused(result);
    unused(invoker);
    SFCError error;

    switch (reason) {
        case AVK_SOFT1: // ߂
            if (feedList != null && _cursor >= 0) {
                rssreader_sfy::SetCurrentFeed((feedList->GetSize() > _cursor) ? ((*feedList)[_cursor]) : null);
                rssreader_sfy::GetMainWindow()->Update();
            }
            GetParent()->Terminate();
            this->Terminate();
            break;
        case AVK_SOFT2: // Tuj[
            if (_listURL->GetItemCount() != 0) {
                error = MakeMenu();
            }
            else {
                // tB[hǉEBhEJ
                if ((error = CreateFrame()) == SFERR_NO_ERROR) {
                    if ((error = CreateInputWindow()) != SFERR_NO_ERROR) {
                        _frame->Terminate();
                    }
                }
            }
            break;
    }
    return;
}

XANDLER_IMPLEMENT_VOIDRESULT(SubscriptionWindow, OnMenuResult, invoker, reason, result)
{
    RSSFeedListPtr feedList = rssreader_sfy::GetRSSFeedList();
    SFCError error;
    unused(reason);

    switch (result) {
        case 0: // ADD
            if ((error = CreateFrame()) == SFERR_NO_ERROR) {
                if ((error = CreateInputWindow()) != SFERR_NO_ERROR) {
                    _frame->Terminate();
                }
            }
            break;
        case 1: // UPDATE
            if (_cursor >= 0) {
                rssreader_sfy::SetCurrentFeed((*feedList)[_cursor]);
                _dialoger->SetTitle(Dialoger::CONFIRM);
                _dialoger->SetMessageText(rssreader_sfy::LoadString(IDS_DIALOG_CONNECT_CONFIRM));
                _dialoger->SetButtonState(Dialoger::OK | Dialoger::CANCEL);
                _dialoger->RegisterHandler(XANDLER_INTERNAL(OnConnect));
                error = _dialoger->Execute();
            }
            break;
        case 2: // DELETE
            if (_listURL->GetItemCount() > 0) {
                _dialoger->SetTitle(Dialoger::CONFIRM);
                _dialoger->SetMessageText(rssreader_sfy::LoadString(IDS_DIALOG_DELETE_CONFIRM));
                _dialoger->SetButtonState(Dialoger::OK | Dialoger::CANCEL);
                _dialoger->RegisterHandler(XANDLER_INTERNAL(OnDelete));
                error = _dialoger->Execute();
            }
            break;
    }
    invoker->Terminate();
    return;
}

// tB[hꗗInh
XANDLER_IMPLEMENT_VOIDRESULT(SubscriptionWindow, OnSelect, invoker, reason, result)
{
    RSSFeedListPtr feedList = rssreader_sfy::GetRSSFeedList();
    unused(invoker);
    unused(reason);
    unused(result);

    _cursor = _listURL->GetCurrentValue();
    if (feedList != null && feedList->GetSize() > _cursor) {
        rssreader_sfy::SetCurrentFeed((*feedList)[_cursor]);
        rssreader_sfy::GetMainWindow()->Update();
        GetParent()->Terminate();
        this->Terminate();
    }
    return;
}

XANDLER_IMPLEMENT_VOIDVALUE(SubscriptionWindow, OnListValue, invoker, reason, value)
{
    unused(invoker);
    RSSFeedListPtr feedList = rssreader_sfy::GetRSSFeedList();

    switch (reason) {
        case SFP16_VALUE_MAXIMUM:
            // ڐ 0 ̂ƂƂȊOŃ\tgL[x؂ւ
            _soft->SetTextKey(
                SOFTMENU_SUBSCRIPTION_WINDOW, SFZSoftKeyControl::SOFTKEY_2,
                (value != 0) ? IDS_SOFTKEY_SUBMENU : IDS_SOFTKEY_ADD
            );
            break;
        case SFP16_VALUE_CURRENT:
            // URL x̃eLXgZbg
            _cursor = value;
            if (feedList != null && _cursor >= 0) {
                if (feedList->GetSize() > _cursor) {
                    if (_labelURL != null) {
                        _labelURL->SetText("URL: " + (*feedList)[_cursor]->_url);
                    }
                }
            }
            break;
    }
    return;
}

// tB[h폜nh
XANDLER_IMPLEMENT_VOIDRESULT(SubscriptionWindow, OnDelete, invoker, reason, result)
{
    unused(invoker);
    unused(result);

    _dialoger->Terminate();
    RSSFeedListPtr feedList = rssreader_sfy::GetRSSFeedList();
    switch(reason) {
        case SFP16_RESULT_OK:
            if (feedList != null) {
                feedList->Delete(_cursor);
                rssreader_sfy::GetRSSFeedList()->SaveToFile();
                Update();
            }
            break;
    }
    return;
}

// ڑmF_CAO nh
XANDLER_IMPLEMENT_VOIDRESULT(SubscriptionWindow, OnConnect, invoker, reason, result)
{
    SFCError error;
    unused(invoker);
    unused(result);

    _dialoger->Terminate();
    if (reason == SFP16_RESULT_OK) {
        if ((error = _http.Open()) == SFERR_NO_ERROR) {
            if ((error = _http.SetMethod("GET")) == SFERR_NO_ERROR) {
                error = _http.Connect((*rssreader_sfy::GetRSSFeedList())[_cursor]->_url, CALLBACK_FUNCTION(ConnectCallback));
                if (error == SFERR_NO_ERROR) {
                    _dialoger->SetTitle(Dialoger::CONNECTING);
                    _dialoger->SetMessageText(rssreader_sfy::LoadString(IDS_DIALOG_CONNECTING));
                    _dialoger->SetButtonState(Dialoger::CANCEL);
                    _dialoger->RegisterHandler(XANDLER_INTERNAL(OnConnecting));
                    error = _dialoger->Execute();
                }
            }
        }
        else {
            _dialoger->SetTitle(Dialoger::CONFIRM);
            _dialoger->SetMessageText(rssreader_sfy::LoadString(IDS_DIALOG_CONNECT_FAIL));
            _dialoger->RegisterHandler(XANDLER_INTERNAL(OnConnecting));
            error = _dialoger->Execute();
        }
    }
    return;
}

// ʐMInh
XANDLER_IMPLEMENT_VOIDRESULT(SubscriptionWindow, OnConnecting, invoker, reason, result)
{
    unused(invoker); unused(reason); unused(result);

    _http.Cancel();
    _http.Close();
    _dialoger->Terminate();
}

// HTTP ڑJnʒmR[obN
XALLBACK_IMPLEMENT_SFXHTTPCONNECTION(SubscriptionWindow, ConnectCallback, error)
{
    if (error == SFERR_NO_ERROR) {
        if (_http.GetResultCode() != 200) {
            _dialoger->Terminate();
            _dialoger->RegisterHandler(XANDLER_INTERNAL(OnConnecting));
            _dialoger->SetTitle(Dialoger::ERROR);
        }
        // ǂݎXg[擾
        switch (_http.GetResultCode()) {
            case 200:
                if ((error = _http.GetStreamReader(1024, &_reader)) == SFERR_NO_ERROR) {
                    // f[^M
                    error = _reader.Fetch(CALLBACK_FUNCTION(FetchCallback));
                }
                break;
            case 400:
                _dialoger->SetMessageText(rssreader_sfy::LoadString(IDS_DIALOG_ERROR_400));
                break;
            case 403:
                _dialoger->SetMessageText(rssreader_sfy::LoadString(IDS_DIALOG_ERROR_403));
                break;
            case 404:
                _dialoger->SetMessageText(rssreader_sfy::LoadString(IDS_DIALOG_ERROR_404));
                break;
            default:
                _dialoger->SetMessageText(rssreader_sfy::LoadString(IDS_DIALOG_ERROR_OTHER));
                break;
        }
        if (_http.GetResultCode() != 200) {
            _dialoger->Execute();
        }
    }
    if (error != SFERR_NO_ERROR) {
        // ʐMI_CAO̕\
        _dialoger->Terminate();
        _dialoger->SetTitle(Dialoger::ERROR);
        _dialoger->SetMessageText(rssreader_sfy::LoadString(IDS_DIALOG_CONNECT_FAIL));
        _dialoger->RegisterHandler(XANDLER_INTERNAL(OnConnecting));
        _dialoger->Execute();
    }

    // R[obN֐ł͖Iɕ`֐Ăяo
    GetRoot()->Render(true);
    return;
}

// HTTP MʒmR[obN
XALLBACK_IMPLEMENT_SFXBINARYSTREAMREADER(SubscriptionWindow, FetchCallback, error)
{
    SFXAnsiString string;
    RSSFeedPtr feed = (*rssreader_sfy::GetRSSFeedList())[_cursor];

    if (error == SFERR_NO_ERROR) {

        _reader.ReadSFXAnsiString(&string);
        _xml += string;

        // MI
        if (_reader.Ends() && _reader.GetReadableSize() == 0) {
            // SFXHTTPConnection ̏I
            _http.Close();

            // ݎ擾
            feed->_date = SFXDate::CurrentDate();
            // 
            error = feed->Parse(_xml);
            _xml.Clear();

            if (error == SFERR_NO_ERROR) {
                // ʂt@Cɕۑ
                feed->SaveToFile();
                // Xgɕۑ
                rssreader_sfy::GetRSSFeedList()->SaveToFile();
            }

            // tB[hXV_CAO̕\
            _dialoger->Terminate();
            _dialoger->SetTitle(Dialoger::DONE);
            _dialoger->SetMessageText(rssreader_sfy::LoadString(IDS_DIALOG_FEED_UPDATED));
            _dialoger->Execute();

            rssreader_sfy::GetMainWindow()->Update();

            // R[obN֐ł͖Iɕ`֐Ăяo
            GetRoot()->Render(true);
        }
        // ܂M\ȃf[^cĂꍇo
        else {
            _reader.Fetch(CALLBACK_FUNCTION(FetchCallback));
        }
    }
    else {
        // ʐMI_CAO
        _dialoger->SetTitle(Dialoger::ERROR);
        _dialoger->SetMessageText(rssreader_sfy::LoadString(IDS_DIALOG_CONNECT_FAIL));
        _dialoger->RegisterHandler(XANDLER_INTERNAL(OnConnecting));
        _dialoger->Execute();

        // R[obN֐ł͖Iɕ`֐Ăяo
        GetRoot()->Render(true);
    }
    return;
}
